<?php

namespace Drupal\simple_xapi;

use Drupal\Component\Serialization\SerializationInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\key\KeyRepositoryInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * SimpleXAPI service.
 */
class SimpleXAPI implements SimpleXAPIInterface {

  /**
   * The serialization.json service.
   *
   * @var \Drupal\Component\Serialization\SerializationInterface
   */
  public $serializationJson;

  /**
   * The key.repository service.
   *
   * @var \Drupal\key\KeyRepositoryInterface
   */
  protected $keyRepository;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The logger.factory service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * Authed User.
   *
   * @var string
   */
  protected $user;

  /**
   * Authed User's password.
   *
   * @var string
   */
  protected $password;

  /**
   * Request Header.
   *
   * @var array
   */
  protected $header = [
    'Content-Type' => 'application/json',
    'X-Experience-API-Version' => '1.0.3',
  ];

  /**
   * Endpoint Settings.
   *
   * @var string
   */
  protected $endpoint;

  /**
   * Debug logging setting.
   *
   * @var bool
   */
  protected $debugLogging;

  /**
   * Error logging setting.
   *
   * @var bool
   */
  protected $errorLogging;

  /**
   * Constructs a SimpleXAPI object.
   *
   * @param \Drupal\Component\Serialization\SerializationInterface $serialization_json
   *   The serialization.json service.
   * @param \Drupal\key\KeyRepositoryInterface $key_repository
   *   The key.repository service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger.factory service.
   */
  public function __construct(
    SerializationInterface $serialization_json,
    KeyRepositoryInterface $key_repository,
    ConfigFactoryInterface $config_factory,
    ClientInterface $http_client,
    LoggerChannelFactoryInterface $logger
  ) {
    $this->serializationJson = $serialization_json;
    $this->keyRepository = $key_repository;
    $this->config = $config_factory->get('simple_xapi.settings');
    $this->httpClient = $http_client;
    $this->logger = $logger->get('simple_xapi');
    $this->endpoint = $this->config->get('endpoint', '');
    $this->debugLogging = $this->config->get('debug_logging_enabled', FALSE);
    $this->errorLogging = $this->config->get('error_logging_enabled', FALSE);
    if (!empty($this->config->get('auth_user_key', ''))) {
      $this->user = $this->keyRepository
        ->getKey($this->config->get('auth_user_key', ''))
        ->getKeyValue();
    }
    else {
      $this->user = '';
    }
    if (!empty($this->config->get('auth_password_key', ''))) {
      $this->password = $this->keyRepository
        ->getKey($this->config->get('auth_password_key', ''))
        ->getKeyValue();
    }
    else {
      $this->password = '';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function sendRequest($action, array $data, $method) {

    if (!in_array($action, ['statements', 'about'])
      || !in_array($method, ['GET', 'POST', 'HEAD'])) {
      $message = str_replace(
        ['%action', '%method'],
        [$action, $method],
        'Wrong action/method! ["statements", "about"]/["GET", "POST", "HEAD"] allowed. "%action"/"%method" was given.');
      return [
        'status' => 403,
        'body' => $message,
      ];
    }
    $json = $this->serializationJson->encode($data);

    $this->header['Content-Length'] = strlen($json);

    if ($this->debugLogging) {
      $this->logger->info(
        'Submitting ' . $method . ' request to "' . $this->endpoint . '/' . $action . '"!'
      );
    }

    try {
      $response = $this->httpClient->request(
        $method, "{$this->endpoint}/{$action}", [
          'auth' => [
            $this->user,
            $this->password,
          ],
          'headers' => $this->header,
          'body' => $json,
        ]
      );
      $output = [
        'status' => $response->getStatusCode(),
        'body' => $response->getBody(),
      ];

    }
    catch (GuzzleException $e) {
      if ($this->errorLogging) {
        $this->logger->error($e->getMessage());
      }
      $output = [
        'status' => $e->getCode(),
        'body' => $e->getMessage(),
      ];

    }

    return $output;
  }

  /**
   * {@inheritdoc}
   */
  public function testConnection() {
    $response = $this->sendRequest('statements', [], 'HEAD');
    if ($response['status'] == 200) {
      $this->logger->info('Endpoint connection check: success!');
    }
    else {
      $this->logger->warning('Endpoint connection check: failed with error: ' . $response['body']);
    }
    return $response;
  }

}
